require "thrift_client/pool"
require "fiber"

class FiberConnectionPool < ConnectionPool

	def initialize(opts, &block)
    super
	end

	# Choose first idle connection and pass it to the supplied
	# block. This will block indefinitely until there is an idle
	# connection to service the request.
	def execute
		f = Fiber.current
		begin
		  conn = acquire(f)
		  yield conn
		ensure
		  release(f)
		end
	end

  # Destroy all connections in this pool. It will waiting until
  # all connections are idle and than close them one by one.
	def destroy
    while @idle.size < @size
    	fiber = Fiber.current
    	@pending.push fiber
    	Fiber.yield
    end
    @idle.each do | conn |
      begin
        conn.disconnect
      rescue Exception => e
        puts "close connection error! #{e.backtrace}"
      end
    end
	end

	private

	# Acquire a lock on a connection and assign it to executing fiber
	# - if connection is idle, pass it back to the calling block
	# - if pool is full, yield the current fiber until connection is idle
	def acquire(fiber)
	  if conn = @idle.pop
	    @active[fiber.object_id] = conn
	    conn
	  else
	  	@pending.push fiber
	    Fiber.yield
	    acquire(fiber)
	  end
	end

	# Release connection assigned to the supplied fiber and
	# resume any other pending connections (which will
	# immediately try to run acquire on the pool)
	def release(fiber)
	  @idle.push(@active.delete(fiber.object_id))

	  if pending = @pending.shift
	    pending.resume
	  end
	end

end
